import 'package:flutter/material.dart';

import 'package:transparent_image/transparent_image.dart';

import 'package:carvita/data/models/vehicle.dart';

class VehicleSummaryCard extends StatelessWidget {
  final Vehicle vehicle;
  final String nextMaintenanceInfo;
  final VoidCallback onTap;

  const VehicleSummaryCard({
    super.key,
    required this.vehicle,
    required this.nextMaintenanceInfo,
    required this.onTap,
  });

  @override
  Widget build(BuildContext context) {
    return Card(
      color: Theme.of(context).colorScheme.surfaceContainerLowest,
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(16)),
      elevation: 4,
      margin: const EdgeInsets.symmetric(vertical: 8),
      child: InkWell(
        onTap: onTap,
        borderRadius: BorderRadius.circular(16),
        child: Padding(
          padding: const EdgeInsets.all(18.0),
          child: Row(
            children: [
              ClipRRect(
                borderRadius: BorderRadius.circular(12),
                child:
                    vehicle.image != null && vehicle.image!.isNotEmpty
                        ? FadeInImage(
                          placeholder: MemoryImage(kTransparentImage),
                          image: MemoryImage(vehicle.image!),
                          fadeInDuration: const Duration(milliseconds: 200),
                          width: 60,
                          height: 60,
                          fit: BoxFit.cover,
                          imageErrorBuilder:
                              (context, error, stackTrace) => Container(
                                width: 60,
                                height: 60,
                                color: Colors.grey[300],
                                child: const Icon(
                                  Icons.directions_car,
                                  color: Colors.grey,
                                ),
                              ),
                        )
                        : Container(
                          width: 60,
                          height: 60,
                          color: Colors.grey[300],
                          child: const Icon(
                            Icons.directions_car,
                            color: Colors.grey,
                          ),
                        ),
              ),
              const SizedBox(width: 15),
              Expanded(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                      vehicle.name,
                      style: TextStyle(
                        color: Theme.of(context).colorScheme.onSurface,
                        fontSize: 16,
                        fontWeight: FontWeight.w500,
                      ),
                      overflow: TextOverflow.ellipsis,
                    ),
                    const SizedBox(height: 5),
                    Text(
                      nextMaintenanceInfo,
                      style: TextStyle(
                        color: Theme.of(
                          context,
                        ).colorScheme.onSurface.withValues(alpha: 0.7),
                        fontSize: 13,
                      ),
                    ),
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
